import logging
import subprocess
import time


class Mutex:

    def __init__(self, host: str) -> None:
        self.host = host
        self.occupant: str = None

    def is_free(self, user_id: str) -> bool:
        try:
            output = subprocess.check_output(
                ['ssh', self.host, 'cat ~/.sync_mutex'],
                stderr=subprocess.DEVNULL).decode()
        except subprocess.CalledProcessError:
            return True
        try:
            words = output.strip().split()
            self.occupant = words[0]
            occupant_ok = self.occupant == user_id
            time_elapsed = time.time() - float(words[1]) > 15
            return occupant_ok or time_elapsed
        except:
            logging.exception('Could not read mutex file')
            return False

    def set(self, user_id: str) -> bool:
        if not self.is_free(user_id):
            return False
        try:
            subprocess.check_output(
                ['ssh', self.host, f'echo "{user_id} {time.time():.3f}" > ~/.sync_mutex'],
                stderr=subprocess.DEVNULL
            )
            return True
        except subprocess.CalledProcessError:
            print('Could not write mutex file')
            return False

    def remove(self) -> None:
        try:
            subprocess.check_output(['ssh', self.host, f'rm ~/.sync_mutex'], stderr=subprocess.DEVNULL)
            return True
        except subprocess.CalledProcessError:
            print('Could not remove mutex file')
            return False



#!/usr/bin/env python3
import argparse
import json
import os
import signal
import socket
import subprocess
import sys
import time
from datetime import datetime
from glob import glob

from livesync import Mutex

hostname = socket.gethostname()
processes: list[subprocess.Popen] = []


def start_process(command: str) -> None:
    proc = subprocess.Popen(command, shell=True, preexec_fn=os.setsid, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    processes.append(proc)


def sync(path: str, target_host: str) -> None:
    if not os.path.isdir(path):
        return
    print(f'start syncing "{path}"')
    EXCLUDES = ['.git/', '__pycache__/', 'gphoto/', '.DS_Store', '.pytest_cache', '.vscode', '.github', 'tests']
    excludes = ' '.join([f'--exclude="{e}"' for e in EXCLUDES])
    rsync = f'rsync --prune-empty-dirs --delete -avz --itemize-changes {excludes} {path}/ {target_host}:{os.path.basename(os.path.realpath(path))}'
    start_process(rsync)
    start_process(f'fswatch -r -l 0.1 -o {path} {excludes} | xargs -n1 -I{{}} {rsync}')


def main():

    parser = argparse.ArgumentParser(description='Repeatedly synchronize local workspace with remote machine')
    parser.add_argument('host', type=str, help='the target host (eg. username@hostname)')
    args = parser.parse_args()

    mutex = Mutex(args.host)
    if not mutex.set(hostname):
        print(f'Target is in use by {mutex.occupant}')
        sys.exit(1)
    with open(glob('*.code-workspace')[0]) as f:
        workspace = json.load(f)
    try:
        for p in workspace['folders']:
            sync(p['path'], args.host)
        while mutex.set(hostname):
            for i in range(100):
                for p in processes:
                    # make stdout non-blocking (https://stackoverflow.com/a/59291466/364388)
                    os.set_blocking(p.stdout.fileno(), False)
                    line = str(p.stdout.readline().decode())
                    # only print files transferred to remote (see itemize-changes rsync option)
                    if line.startswith('<'):
                        print(f'{datetime.now().strftime("%X")} deploying: {line[10:].strip()}', flush=True)
                time.sleep(0.1)
        else:
            print(f'Target is in use by {mutex.occupant}')
    except KeyboardInterrupt:
        mutex.remove()
    finally:
        print('stopping sync')
        for p in processes:
            try:
                os.killpg(os.getpgid(p.pid), signal.SIGTERM)
            except:
                pass


if __name__ == '__main__':
    main()



#!/usr/bin/env python
import os
from distutils.core import setup
from pathlib import Path

long_description = (Path(__file__).parent / "README.md").read_text()
VERSION = os.getenv('VERSION', '0.0.1')

setup(
    name='LiveSync',
    version=VERSION,
    description='Repeatedly synchronize local workspace with a (slow) remote machine',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='MIT',
    author='Zauberzeug GmbH',
    author_email='info@zauberzeug.com',
    url='https://github.com/zauberzeug/livesync',
    keywords='snyc remote watch filesystem development deploy live hot reload',
    python_requires='>=3.7',
    packages=['livesync'],
    entry_points={
        'console_scripts': [
            'livesync=livesync.livesync:main',
        ],
    },
)
